# Document.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os.path
import errno

from gtk import *
from gnome.ui import *
from gnome.util import *

import Moleskine
from gtkscintilla import *
import GTKSCINTILLA

class Document(GtkScintilla):
    def __init__(self, name = 'Unnamed'):
        GtkScintilla.__init__(self)
        
        self.connect('save_point_left', self.__save_point_left)
        self.connect('save_point_reached', self.__save_point_reached)
        
    # Hardcoded settings
        self.set_margin_width(1, 0)
        self.set_margin_width(2, 0)
        self.set_caret_period(250)
    # End of hardcoded settings
        
        self.setup_default_style()
        self.setup_line_numbers_style()
        self.setup_line_numbers_digits()
        self.set_language(Moleskine.app.langs['Plain text'])
        
        self.is_modified = self.get_modify
        self.filepath = None
        self.filename = None
        
        self.label = GtkLabel(name)
        
        self.set_usize(320, 240)
    
    def setup_default_style(self):
        self.style_set_font(GTKSCINTILLA.STYLE_DEFAULT,
                            Moleskine.app.prefs['default style/font'])
        self.style_set_fore(GTKSCINTILLA.STYLE_DEFAULT,
                            int(Moleskine.app.prefs['default style/fore']))
        self.style_set_back(GTKSCINTILLA.STYLE_DEFAULT,
                            int(Moleskine.app.prefs['default style/back']))
        self.set_sel_back(TRUE,
                          int(Moleskine.app.prefs['default style/selection']))
        self.marker_set_back(TRUE,
                             int(Moleskine.app.prefs['default style/bookmark']))
    
    def setup_line_numbers_style(self):
        self.style_set_font(GTKSCINTILLA.STYLE_LINE_NUMBER,
                            Moleskine.app.prefs['line numbers/font'])
        self.style_set_fore(GTKSCINTILLA.STYLE_LINE_NUMBER,
                            int(Moleskine.app.prefs['line numbers/fore']))
        self.style_set_back(GTKSCINTILLA.STYLE_LINE_NUMBER,
                            int(Moleskine.app.prefs['line numbers/back']))
    
    def setup_line_numbers_digits(self):
        def digits_width(digits):
            font = load_font(Moleskine.app.prefs['line numbers/font'])
            return font.width('0' * (digits + 1))
        
        digits = 0
        if int(Moleskine.app.prefs['line numbers/show']):
            if int(Moleskine.app.prefs['line numbers/auto']):
                digits = len(str(self.get_line_count())) + 1
            else:
                digits = int(Moleskine.app.prefs['line numbers/digits'])
            self.set_margin_width(0, digits_width(digits))
        else:
            self.set_margin_width(0, 0)
    
    def load_file(self, filepath):
        def read(filepath):
            file = open(filepath, 'r')
            text = file.read()
            file.close()
            return text
        
        if os.path.exists(filepath):
            text = read(filepath)
            self.set_text(text)
        self.filepath = filepath
        self.filename = os.path.basename(filepath)
        self.label.set_text(self.filename)
        
        self.setup_line_numbers_digits()
        self.check_file_types()
        self.empty_undo_buffer()
        self.set_save_point()
    
    def save_file(self, filepath=None):
        def write(filepath, text):
            file = open(filepath, 'w')
            file.write(text)
            file.close()
        
        text = self.get_text()
        if filepath is not None:
            self.filepath = filepath
            self.filename = os.path.basename(filepath)
            self.label.set_text(self.filename)
        write(self.filepath, text)
        self.check_file_types()
        self.set_save_point()
    
    def revert(self):
        if self.filepath is not None:
            self.load_file(self.filepath)
    
    def matches(self, lang):
        import re
        from fnmatch import fnmatch
        
        first_line = self.get_line(0)
        for type, rule in lang.rules:
            if type == 0:
                if fnmatch(self.filename, rule):
                    return TRUE
            elif rule == 1:
                if re.match(rule, filename):
                    return TRUE
            else:
                if re.match(rule, first_line):
                    return TRUE
        return FALSE
    
    def check_file_types(self):
        for lang in Moleskine.app.langs.values():
            if self.matches(lang):
                self.set_language(lang)
    
    def can_close(self):
        return not self.is_modified()
    
    def is_empty(self):
        return self.get_text_length() == 0
    
    def set_language(self, language):
        self.language = language
        
        self.set_tab_width(language.tab_size)
        self.set_use_tabs(not language.soft_tabs)
        self.set_indent_width(language.indent_size)
        self.set_indent_visible(language.show_indent)
        self.set_auto_indent(language.auto_indent)
        self.set_edge_mode(language.edge_indicator)
        self.set_edge_column(language.edge_column)
        self.set_edge_color(language.edge_color)
        
        self.set_lexer(language.lexer)
        self.clear_document_style()
        
        for kwclass in range(5):
            if language.keywords[kwclass] is not None:
                self.set_keywords(kwclass, language.keywords[kwclass])
        
        for i in language.styles.keys():
            self.set_font(i)
            self.set_fore(i)
            self.set_back(i)
        
        self.colourise(0, -1)
    
    def set_font(self, style):
        def merge_font(font):
            import string
            
            df = string.split(Moleskine.app.prefs['default style/font'], '-')
            sf = string.split(font, '-')
            for i in range(min(len(df), len(sf))):
                if sf[i] == '*':
                    sf[i] = df[i]
            return string.join(sf, '-')
        
        value = self.language.styles[style]['font']
        if value is not None:
            self.style_set_font(style, merge_font(value))
        else:
            self.style_set_font(style,
                    Moleskine.app.prefs['default style/font'])
    
    def set_fore(self, style):
        value = self.language.styles[style]['fore']
        if value is not None:
            self.style_set_fore(style, int(value))
        else:
            self.style_set_fore(style,
                                int(Moleskine.app.prefs['default style/fore']))
    
    def set_back(self, style):
        value = self.language.styles[style]['back']
        if value is not None:
            self.style_set_back(style, int(value))
        else:
            self.style_set_back(style,
                                int(Moleskine.app.prefs['default style/back']))
    
    def reload_language(self):
        self.set_language(self.language)
    
    def set_auto_indent(self, do_indent):
        if do_indent:
            self.__char_added_signal = self.connect('char_added',
                                                    self.auto_indent)
        else:
            try:
                self.disconnect(self.__char_added_signal)
                self.__char_added_signal = None
            except AttributeError:
                pass
    
    def auto_indent(self, w, char, user_data=None):
        if char == 10 or char == 13:
            from string import strip
            
            line = self.get_current_line() - 1
            while line > 0 and strip(self.get_line(line)) == '':
                line = line - 1
            if line > 0:
                indent = self.get_line_indentation(line)
                line = self.get_current_line()
                self.begin_undo_action()
                self.set_line_indentation(line, indent)
                self.go_to_pos(self.get_line_indent_position(line))
                self.end_undo_action()
    
    def get_current_line(self):
        return self.line_from_position(self.get_current_pos())
    
    def __save_point_left(self, w, user_data=None):
        if self.filename is not None:
            self.label.set_text('*' + self.filename)
        else:
            self.label.set_text('*Unnamed')
        Moleskine.app.update_title()
    
    def __save_point_reached(self, w, user_data=None):
        if self.filename is not None:
            self.label.set_text(self.filename)
        else:
            self.label.set_text('Unnamed')
        Moleskine.app.update_title()
