# MoleskineApp.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os.path

from gtk import *
from gnome.ui import *
from gnome.uiconsts import *

from Document import Document
from Config import Config
from Languages import LanguagesList
import menu
import toolbar

class MoleskineApp(GnomeApp):
    def __init__(self):
        GnomeApp.__init__(self, 'GnomeMoleskine', 'Moleskine')
        
        self.switch = TRUE
        self.documents = []
        self.load_prefs()
        
        self.notebook = GtkNotebook()
        self.notebook.connect_after('switch-page', self.__switch_page)
        self.notebook.set_scrollable(TRUE)
        self.notebook.show()
        
        self.set_contents(self.notebook)
        self.set_wmclass('main_window', 'Moleskine')
        self.set_geometry_hints(min_width=0, min_height=0)
        if int(self.prefs['keyboard/keybindings']) == 2:
            import Emacs
            self.create_menus(Emacs.main_menu)
            self.connect('key_press_event', Emacs.key_press)
        elif int(self.prefs['keyboard/keybindings']) == 1:
            import GnomeMenu
            self.create_menus(GnomeMenu.main_menu)
        else:
            self.create_menus(menu.main_menu)
        self.create_toolbar(toolbar.main_toolbar)
        self.connect('delete_event', menu.exit)
        self.show_all()
        
        self.load_langs()
        self.load_mru()
    
    def new_document(self):
        """Create a new document."""
        doc = Document()
        self.documents.append(doc)
        self.notebook.append_page(doc, doc.label)
        self.notebook.set_page(-1)
    
    def open_document(self, filepath):
        """Open a document.
        
        The file name can contain relative paths, special shell chars (~) and
        environment variables.
        If the current document is empty the newly created one will replace it.
        If the given file doesn't exist you will be asked about to create it.
        """
        
        filepath = os.path.expanduser(filepath)
        filepath = os.path.expandvars(filepath)
        filepath = os.path.abspath(filepath)
        
        if os.path.isdir(filepath):
            GnomeMessageBox(
                'Cannot open `%s\': is a directory' % \
                os.path.basename(filepath),
                MESSAGE_BOX_ERROR, STOCK_BUTTON_OK).run_and_close()
        elif not os.path.exists(filepath):
            res = GnomeMessageBox(
                    'File `%s\' not found.\nDo you want to create it?' % \
                            os.path.basename(filepath),
                    MESSAGE_BOX_QUESTION,
                    STOCK_BUTTON_YES, STOCK_BUTTON_NO).run_and_close()
            if res == 0:
                self.__load(filepath)
                self.get_current_document().set_text(' ')
                self.get_current_document().set_text('')
            else:
                if len(self.documents) == 0:
                    self.new_document()
        else:
            self.__load(filepath)
        self.get_current_document().grab_focus()
    
    def __load(self, filepath):
        doc = self.get_current_document()
        
        if doc is None or not doc.is_empty():
            self.new_document()
            doc = self.get_current_document()
        doc.load_file(filepath)
        self.add_to_mru(filepath)
    
    def save_document(self, filepath = None):
        self.get_current_document().save_file(filepath)
    
    def close_document(self):
        """Close the active document.
        
        If the document to be closed is the last one, a new empy document will
        be created.
        If the document has been modified and not saved you will be asked if
        you want to save the changes.
        """
        doc = self.get_current_document()
        if not doc.can_close():
            res = GnomeMessageBox(
                    'File `%s\' has been modified\nand not saved.'
                    ' Do you want to save it now?' % doc.filename,
                    MESSAGE_BOX_QUESTION, STOCK_BUTTON_YES, 
                    STOCK_BUTTON_NO, STOCK_BUTTON_CANCEL).run_and_close()
            if res == 0:
                doc.save_file()
            elif res == 2 or res < 0:
                return
        page = self.notebook.get_current_page()
        self.notebook.remove_page(page)
        self.documents.remove(doc)
        if len(self.documents) == 0:
            self.new_document()
        else:
            self.notebook.set_page(page)
    
    def ask_file(self, type, ok_cb):
        if type == 'load':
            title = 'Open document...'
            wm_name = 'open_dialog'
        elif type == 'save':
            title = 'Save as...'
            wm_name = 'save_dialog'
        else:
            title = type
            wm_name = 'file_dialog'
        
        fs = GtkFileSelection(title)
        fs.set_wmclass(wm_name, 'Moleskine')
        fs.set_transient_for(self)
        
        doc = self.get_current_document()
        if doc is not None and doc.filepath is not None:
            startdir = os.path.dirname(doc.filepath)
            fs.set_filename(startdir + '/')
        if ok_cb is not None:
            fs.ok_button.connect_object('clicked', ok_cb, fs)
        
        fs.ok_button.connect_object('clicked', GtkWidget.destroy, fs)
        fs.cancel_button.connect_object('clicked', GtkWidget.destroy, fs)
        fs.show()
    
    def get_current_document(self):
        """Returns the currently active document.
        
        If there are no documents (should never happen) it will return None.
        """
        if len(self.documents) > 0:
            return self.documents[self.notebook.get_current_page()]
        else:
            return None
    
    def load_mru(self):
        self.mru = Config('Moleskine/PreviousFiles')
        self.mru.load()
        
        self.files = []
        if self.mru.has_key('files/count'):
            for i in range(int(self.mru['files/count'])):
                self.files.append(self.mru['files/file_%i' % i])
        self.update_mru_menu()
    
    def save_mru(self):
        self.files = self.files[:int(self.prefs['history/mru_size'])]
        for i in range(len(self.files)):
            self.mru['files/file_%i' % i] = self.files[i]
        self.mru['files/count'] = len(self.files)
        self.mru.save()
    
    def update_mru_menu(self):
        import copy, string
        
        fix_underscore = lambda label, replace=string.replace: \
                replace(label, '_', '__')
        
        build_item = lambda label: \
                (APP_UI_ITEM, label, None,
                 menu.open_recent, None,
			     APP_PIXMAP_NONE, None, 0, 0)
        
        temp_list = copy.copy(self.files)[:int(self.prefs['history/mru_size'])]
        if int(self.prefs['history/mru_sort']):
            temp_list.sort()
        temp_list = map(fix_underscore, temp_list)
        mru_menu = map(build_item, temp_list)
        self.remove_menus('File/Open previous/', 55)
        self.insert_menus('File/Open previous/', mru_menu)
    
    def add_to_mru(self, filepath):
        if filepath in self.files:
            self.files.remove(filepath)
        self.files.insert(0, filepath)
        if len(self.files) > int(self.prefs['history/mru_size']):
            self.files = self.files[:int(self.prefs['history/mru_size'])]
        self.update_mru_menu()
    
    def load_prefs(self):
        self.prefs = Config('Moleskine/Preferences')
        self.prefs.load()
        
        w = int(self.prefs['geometry/width'])
        h = int(self.prefs['geometry/height'])
        self.set_default_size(w, h)
    
    def save_prefs(self):
        if int(self.prefs['geometry/save_win_size']):
            self.prefs['geometry/width'] = self.get_window().width
            self.prefs['geometry/height'] = self.get_window().height
    
    # FIXME: Can't get the right window position! :( This gives
    # the position of the client window (without decorations)
    # and set_uposition places the window with deocorations. Hints?
    #    if int(self.prefs['geometry/save_win_pos']):
    #        self.prefs['geometry/pos_x'] = self.get_window().x
    #        self.prefs['geometry/pos_y'] = self.get_window().y
        self.prefs.save()
    
    def load_langs(self):
        self.langs = LanguagesList()
        self.langs.load()
        
        langs_names = self.langs.keys()
        langs_names.sort()
        build_item = lambda label: \
                (APP_UI_ITEM, label, None,
                 menu.set_lang, None,
			     APP_PIXMAP_NONE, None, 0, 0)
        langs_menu = [UIINFO_RADIOLIST(map(build_item, langs_names))]
        self.insert_menus('Settings/Change language/', langs_menu)
    
    def update_styles(self):
        for doc in self.documents:
            doc.setup_default_style()
            doc.setup_line_numbers_style()
            doc.reload_language()
    
    def update_title(self):
        if self.switch == FALSE:
            return
        doc = self.get_current_document()
        if doc.filepath is None:
            title = 'Unnamed'
        else:
            title = doc.filename
        if doc.is_modified():
            sep = ' * '
        else:
            sep = ' - '
        self.set_title('Moleskine' + sep + title)
    
    def __switch_page(self, notebook, notebook_page, page_num, data=None):
        if self.switch == FALSE:
            return
        doc = self.get_current_document()
        self.update_title()
        try:
            langs_names = self.langs.keys()
            langs_names.sort()
            index = langs_names.index(doc.language.name)
            menu_items = self.langs_menu.children()
            if len(langs_names) < len(menu_items):
                menu_items = menu_items[1:]
            menu_items[index].set_active(TRUE)
        except AttributeError:
            pass
        doc.grab_focus()
